# $Id: map.py,v 1.59 2011/02/15 23:24:08 irees Exp $
import copy
import collections

import emen2.db.config
g = emen2.db.config.g()
from emen2.web.view import View


@View.register
class Sitemap(View):

	@View.add_matcher(r'^/sitemap/$', r'^/sitemap/(?P<recid>\w+)/$')
	def init(self, recid=None, recurse=-1):
		self.template = '/pages/page'
		
		if recid == None:
			recid = g.BOOKMARKS.get("GROUPS", 0)
		
		sm = Map(mode="both", keytype="record", root=int(recid), db=self.db, recurse=recurse)
		smdata = sm.get_data()
		self.title = 'Site Map'
		self.set_context_item("content", smdata)
		
		

@View.register
class Map(View):

	@View.add_matcher(r'^/map/(?P<keytype>\w+)/(?P<root>\w+)/(?P<mode>\w+)/$')
	def init(self, keytype="record", root=None, mode="parents", rctx=0, recurse=-1, sparse=1, collapsedrectypes=[], maxnodes=100000, showroot=1):

		self.template = '/pages/map'
		self.keytype = keytype
		self.root = root
		self.mode = mode
		self.rctx = rctx
		self.showroot = int(showroot)
		self.sparse = sparse
		self.maxnodes = maxnodes
		self.recnames = {}

		self.recurse = int(recurse)
		self.collapsed = collections.defaultdict(set)
		self.collapsedchildren = {}
		self.collapsedrectypes = collapsedrectypes
		if not self.collapsedrectypes and self.mode in ["both", "children"]:
			self.collapsedrectypes = ["scan", "ccd", "ccd_jadas", "micrograph", "grid", "stackimage"]

		if self.keytype == "record":
			self.root = int(self.root)

		# try:
		if self.mode == "both":
			table, img = self.build_both()
		else:
			table, img = self.build(mode=self.mode)

		# except Exception, inst:
		# 	self.template='/errors/error_mapoverride'
		# 	self.ctxt["confirm"] = 0
		# 	if isinstance(inst,ValueError):
		# 		self.ctxt["confirm"] = 1
		# 	self.ctxt["mode"] = self.mode
		# 	self.ctxt["keytype"] = self.keytype
		# 	self.ctxt["root"] = self.root
		# 	self.ctxt["errmsg"] = inst
		# 	return


		self.ctxt["root"] = self.root
		self.ctxt["table"] = table
		self.ctxt["img"] = img
		self.ctxt["recnames"] = self.recnames
		self.ctxt["keytype"] = self.keytype
		self.ctxt["mode"] = self.mode
		self.ctxt["collapsed"] = self.collapsed


	def build_both(self):
		table1, img1 = self.build(mode="parents")
		table2, img2 = self.build(mode="children")

		y1, x1 = len(table1), len(table1[0])
		y2, x2 = len(table2), len(table2[0])

		# Pad length
		if x1 < 2:
			for y in range(y1):
				table1[y].insert(0, None)
				img1[y].insert(0, '')
			x1 = len(table1[0])
		
		if x2 < 2:
			for y in range(y2):
				table2[y].append(None)
				img2[y].append('')
			x2 = len(table2[0])

		# Pad height
		h = y1
		if y2 > h:
			h = y2

		for y in range(h-y1):
			table1.append([None]*x1)
			img1.append([""]*x1)

		for y in range(h-y2):
			table2.append([None]*x2)
			img2.append([""]*x2)

		for y in range(h):
			table1[y].extend(table2[y][1:])
			img1[y].extend(img2[y][:-1])


		return table1, img1



	def bfs(self, root, tree):
		stack = copy.copy(tree.get(root, set()))
		seen = set([root])
		seen |= stack
		while stack:
			cur = stack.pop()
			children = tree.get(cur, set())
			stack |= children
			seen |= children
		return seen

		


	def build(self, mode):
		
		if mode == "children":
			results = self.db.getchildtree(self.root, keytype=self.keytype, recurse=self.recurse)
		else:
			results = self.db.getparenttree(self.root, keytype=self.keytype, recurse=self.recurse)		
		

		if len(results) > self.maxnodes and self.maxnodes:
			raise ValueError, """There are %s records in this map, which exceeds the safety limit of %s; are you sure you want to render this? It will likely crash your browser."""%(len(results),self.maxnodes)

		r = [self.root]
		for i in results.values():
			r.extend(i)
		r = set(r)

		if self.keytype=="record":
			rectypes = self.db.groupbyrecorddef(r)
			colrecs = []
			for i in self.collapsedrectypes:
				colrecs.extend(rectypes.get(i,[]))
			colrecs = set(colrecs)
			uncollapsed = r - colrecs
			self.recnames.update(self.db.renderview(uncollapsed, viewtype="recname"))

		else:
			if self.keytype=="paramdef":
				items = self.db.getparamdef(r)
			else:
				items = self.db.getrecorddef(r)

			for i in items:
				self.recnames[i.name] = i.desc_short or i.name # "%s<br />%s"%(i.name,i.desc_short)


		depth = self.finddepth(self.root, results)
		table = [[None]*(depth+1)]
		table[0][0] = self.root
		img = [[""]*(depth+1)]

		y = 0
		x = 0
		while y < len(table):
	
			x = 0
			while x < depth:

				pos = table[y][x]
				children = copy.copy(results.get(pos,set()))
		
				# This is a bit ugly
				if mode == "children" and  self.keytype == "record" and self.collapsedrectypes:
					c = colrecs & children
					if c:
						for i in self.collapsedrectypes:
							cc = rectypes.get(i, set()) & c
							if len(cc) > 0:
								
								pruned = set()
								for j in cc:
									pruned |= self.bfs(j, results)
									
								rep = sorted(cc)[0] #cc.pop()
								children -= pruned
								children.add(rep)								
								self.collapsed[rep] |= pruned
								pruned -= cc

								if pruned:
									self.recnames[rep] = "%s %ss, and %s others"%(len(cc), i, len(pruned))
								else:
									self.recnames[rep] = "%s %ss"%(len(cc), i)
								
								results[rep] = set()

									
				self.collapsedchildren[pos] = children
				
				
				# Sort by recname...
				for yoffset,child in enumerate(sorted(children, key=lambda x:self.recnames.get(x,'').lower())):
					if yoffset == 0:
						table[y][x+1] = child
					if yoffset > 0:
						table.insert(y+yoffset,[None]*(depth+1))
						table[y+yoffset][x+1] = child

		
				if len(children) > 0:
					img[y][x]="next"
					
				if len(children) > 1:
					img[y][x]="branch_next"
					for yoffset in range(len(children))[1:]:
						img.insert(y+yoffset,[""]*(depth+1))
						img[y+yoffset][x] = "branch_both"
					img[y+yoffset][x] = "branch_up"
		
				x+=1		
			y+=1

		# add spacers; paints down columns looking for gaps
		paint=0
		for x in range(len(img[0])):
			for y in range(len(img)):
				if img[y][x]:
					paint = 0
				if paint:
					img[y][x]="branch_spacer"		
				if img[y][x] in ["branch_next","branch_both"]:
					paint = 1


		if mode == "parents":
			for i in table:
				i.reverse()
			for i in img:
				i.reverse()
				
		if mode == "children":
			for y in range(len(img)):
				for x in range(len(img[0])):
					img[y][x] = img[y][x]+"_reverse"

		return table, img



	def finddepth(self, root, graph):

		depth=0

		v=[]
		for i in graph.values():
			v.extend(i)
		v=set(v)

		f1=graph.get(root,set())
		while f1 & v:

			#if depth > self.maxrecurse:
			#	raise Exception,"Recursion depth exceeded (circular relationship?)"

			f2=set()
			for i in f1:
				f2 |= graph.get(i,set())
			f1=f2
			depth+=1

		return depth












__version__ = "$Revision: 1.59 $".split(":")[1][:-1].strip()
