# $Id: proxy.py,v 1.4 2011/02/10 21:32:48 edwlan Exp $
import copy
import urllib
import urlparse
import itertools
from hashlib import sha1
import random
import time
import UserDict, collections
collections.Mapping.register(UserDict.DictMixin)

import emen2.util.jsonutil

class JSONRPCException(Exception):
	def __init__(self, rpcError):
		Exception.__init__(self, rpcError.get('message'))
		self.data = rpcError.get('data')
		self.message = rpcError.get('message')
		self.code = rpcError.get('code')

inst = lambda x:x()
class JSONRPCProxy(object):
	@inst
	class IDGen(object):
		def __init__(self):
			self.__hasher = sha1()
			self.__id = 0
		def __get__(self, *_, **__):
			self.__id += 1
			self.__hasher.update(str(self.__id))
			self.__hasher.update(time.ctime())
			self.__hasher.update(str(random.random))
			return self.__hasher.hexdigest()

	@classmethod
	def from_url(cls, url, ctxid=None, serviceName=None):
		urlsp = urlparse.urlsplit(url)
		url = '%s://%s' % (urlsp.scheme, urlsp.netloc)
		path = urlsp.path
		if urlsp.query: path = '%s?%s' % (path, urlsp.query)
		if urlsp.fragment: path = '%s#%s' % (path, urlsp.fragment)
		return cls(url, path, serviceName, ctxid)

	def transform_url(*a): return a

	def __init__(self, host, path='/jsonrpc', serviceName=None, *args, **kwargs):
		self.serviceURL = host
		self.__serviceName = serviceName
		self.__path = path
		self.init(*args, **kwargs)
		self.serviceURL, self.__path = self.transformURL(host, path)

	def transformURL(self, serviceURL, path):
		return serviceURL, path
	def init(self, *args, **kwargs): pass

	def __getattr__(self, name):
		if self.__serviceName != None:
			name = "%s.%s" % (self.__serviceName, name)
		return self.__class__(self.serviceURL, path=self.__path, serviceName=name)

	def get_postdata(self, args, kwargs):
		if kwargs.has_key('__args'): raise ValueError, 'invalid argument name: __args'
		kwargs['__args'] = args or ()
		postdata = emen2.util.jsonutil.encode({
			"method": self.__serviceName,
			'params': kwargs,
			'id': self.IDGen, # get a request id
			'jsonrpc': '2.0'
		})
		return postdata

	def proc_response(self, data): return data

	def __call__(self, *args, **kwargs):
		url = '%(host)s%(path)s' % dict(host = self.serviceURL, path = self.__path)
		postdata = self.get_postdata(args, kwargs)
		respdata = urllib.urlopen(url, postdata).read()
		resp = emen2.util.jsonutil.decode(respdata)

		if resp.get('error') != None:
			raise JSONRPCException(resp['error'])
		else:
			resp = self.proc_response(resp)
			result = resp['result']
			return result

	def batch_call(self, names, *params):
		methods = ( (getattr(self, name),param) for name,param in itertools.izip(names, params) )
		data = (method.get_postdata(params) for method, params in methods)
		postdata = '[%s]' % ','.join(data)
		respdata = urllib.urlopen(self.serviceURL, postdata).read()
		resp = emen2.util.jsonutil.decode(respdata)
		return [res.get('result', res.get('error')) for res in resp]

__version__ = "$Revision: 1.4 $".split(":")[1][:-1].strip()
